﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using iPodm3uCreator.BL;

namespace iPodm3uCreator
{
    public partial class MainForm : Form
    {
        private MusicTreeController musicController;
        
        public MainForm()
        {
            InitializeComponent();
            musicController = new MusicTreeController(musicTreeView);
        }

        private void btnOpenDir_Click(object sender, EventArgs e)
        {
            DialogResult dialogresult = folderBrowserDialog.ShowDialog();
          
            folderBrowserDialog.Description = "Select Music Folder";
            string folderPath = "";
            if (dialogresult == DialogResult.OK)
            {
                folderPath = folderBrowserDialog.SelectedPath;
            }
            musicFolderTextBox.Text = folderPath;

            if (!string.IsNullOrEmpty(musicFolderTextBox.Text))
            {
                musicController.LoadMusicFolder(musicFolderTextBox.Text);
                treeRefreshButton.Enabled = true;
                makePlaylistGroupBox.Enabled = true;
                deletePlaylistsGroupBox.Enabled = true;
                string folderName = musicController.GetName(folderPath);
                rootPlaylistTextBox.Text = folderPath + "\\" + folderName + ".m3u";
            }
        }

        private void splitContainer1_SplitterMoved(object sender, SplitterEventArgs e)
        {
            ((SplitContainer) sender).SplitterDistance = 120;
        }

        private void musicTreeView_AfterExpand(object sender, TreeViewEventArgs e)
        {
            e.Node.ImageIndex = 1;
        }

        private void musicTreeView_AfterCollapse(object sender, TreeViewEventArgs e)
        {
            e.Node.ImageIndex = 0;
        }

        private void musicTreeView_AfterCheck(object sender, TreeViewEventArgs e)
        {
            ((TreeView) sender).AfterCheck -= musicTreeView_AfterCheck;
            bool isChecked = e.Node.Checked;
            CheckAllSubnodes(e.Node, isChecked);
            ((TreeView)sender).AfterCheck += musicTreeView_AfterCheck;
        }

        private void CheckAllSubnodes(TreeNode node, bool isChecked)
        {
            node.Checked = isChecked;
            foreach (TreeNode subNode in node.Nodes)
            {
                CheckAllSubnodes(subNode, isChecked);
            }

        }

        private void makePlaylistButton_Click(object sender, EventArgs e)
        {
            List<string> fileList = musicController.GetCheckedFilesWithPath();

            try
            {
                #region structure.txt
                if (fileList.Count > 0)
                {
                    string structureFileName = musicController.MusicFolder + "\\structure.txt";

                    try
                    {

                        //structure.txt file
                        if (File.Exists(structureFileName))
                        {
                            File.Delete(structureFileName);
                        }
                        if (makeStructureCheckBox.Checked)
                        {
                            File.WriteAllLines(structureFileName, fileList.ToArray());
                        }

                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(ex.Message);
                    }
                }
                #endregion
                
                //root playlist
                string m3uFileName = rootPlaylistTextBox.Text;
                CreatePlaylistFile(m3uFileName, fileList);

                if (makeInAllSubfoldersRadioButton.Checked)
                {
                    //subfolder playlists
                    CreatePlaylistsRecursively(musicController.MusicFolder, musicController.RootNode, 1);
                }
                MessageBox.Show("Playlist(s) created");
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            
        }

        private void CreatePlaylistsRecursively(string startDir, TreeNode startNode, uint startLevel)
        {
            //string dir = startDir + "\\" + startNode.Text;
            startLevel++;
            foreach (TreeNode node in startNode.Nodes)
            {
                bool nodeHaveSubfolders = false;
                foreach (TreeNode subNode in node.Nodes)
                {
                    if (subNode.ImageIndex != 2)
                    {
                        nodeHaveSubfolders = true;
                        break;
                    }
                }
                if (nodeHaveSubfolders)
                {
                    List<string> fileList = musicController.GetCheckedFilesWithPath(node, startLevel);
                    CreatePlaylistFile(startDir + "\\" + node.Text + "\\" + node.Text + ".m3u", fileList);
                    CreatePlaylistsRecursively(startDir + "\\" + node.Text, node, startLevel);
                }
            }
        }
        
        private void CreatePlaylistFile(string fileNameWithPath, List<string> fileList)
        {
            if (fileList.Count > 0)
            {
                
                try
                {
                    List<string> playlistStrings = new List<string>();
                    playlistStrings.Add("#EXTM3U");
                    playlistStrings.AddRange(fileList);

                    //playlist file);
                    if (File.Exists(fileNameWithPath))
                    {
                        File.Delete(fileNameWithPath);
                    }
                    File.WriteAllLines(fileNameWithPath, playlistStrings.ToArray());
                    
                }
                catch (Exception ex)
                {
                    throw ex;
                }
            }
        }

        private void Form1_Resize(object sender, EventArgs e)
        {
            this.splitContainer1.SplitterDistance = 120;
            this.panel2.Height = this.Height - this.panel1.Height - this.panel3.Height - 40;

        }

        private void treeRefreshButton_Click(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(musicFolderTextBox.Text))
            {
                musicController.LoadMusicFolder(musicFolderTextBox.Text);
            }
        }

        private void deletePlaylists_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Do you really want to delete playlists?", "Confirm delete", MessageBoxButtons.YesNo) == DialogResult.Yes)
            {
                string rootFolder = musicController.MusicFolder;
                string rootFolderName = musicController.GetName(rootFolder);
                string rootPlaylistFile = rootFolder + "\\" + rootFolderName + ".m3u";

                try
                {
                    if (File.Exists(rootPlaylistFile))
                    {
                        File.Delete(rootPlaylistFile);
                    }

                    if (deleteStructureFileCheckBox.Checked)
                    {
                        if (File.Exists(rootFolder + "\\structure.txt"))
                        {
                            File.Delete(rootFolder + "\\structure.txt");
                        }
                    }

                    if (deleteAllRadioButton.Checked || deleteAllGeneratedRadioButton.Checked)
                    {
                        DeletePlaylistsRecursively(rootFolder);
                    }

                    MessageBox.Show("Playlist(s) deleted");
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }

            }
        }

        private void DeletePlaylistsRecursively(string folder)
        {
            string[] files;

            if (deleteAllGeneratedRadioButton.Checked)
            {
                files = Directory.GetFiles(folder, musicController.GetName(folder) + ".m3u");
            }
            else
            {
                files = Directory.GetFiles(folder, "*.m3u");
            }
            foreach (string file in files)
            {
                if (File.Exists(file))
                {
                    File.Delete(file);
                }
            }

            string[] subFolders = Directory.GetDirectories(folder);
            foreach (string subFolder in subFolders)
            {
                DeletePlaylistsRecursively(subFolder);
            }

        }
    }
}
